/**
 * @file
 * VuoList_VuoPoint4d C type definition.
 *
 * @copyright Copyright © 2012–2016 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoPoint4d_H
#define VuoList_VuoPoint4d_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoPoint4d VuoList_VuoPoint4d
 * A list of @ref VuoPoint4d elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoPoint4d_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoPoint4d elements.
 */
typedef const struct VuoList_VuoPoint4d_struct { void *l; } * VuoList_VuoPoint4d;
/// @}
#endif

/**
 * Creates a new list of @ref VuoPoint4d elements.
 */
VuoList_VuoPoint4d VuoListCreate_VuoPoint4d(void);

/**
 * Creates a new list of `count` instances of `value`.
 *
 * Use this in conjunction with @ref VuoListGetData_VuoPoint4d to quickly initialize a large list.
 */
VuoList_VuoPoint4d VuoListCreateWithCount_VuoPoint4d(const unsigned long count, const VuoPoint4d value);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoPoint4d VuoListCopy_VuoPoint4d(const VuoList_VuoPoint4d list);

/**
 * Returns the @ref VuoPoint4d at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoPoint4d VuoListGetValue_VuoPoint4d(const VuoList_VuoPoint4d list, const unsigned long index);

/**
 * Returns a pointer to a C array containing the list items.
 *
 * Use this if you need a fast way to get or change multiple list items.
 *
 * You can modify values in the list by changing them in this array.
 * Just don't attempt to access beyond the list size.
 *
 * The pointer becomes invalid if you modify the list size using other functions
 * (e.g., Insert, Prepend, Append, Cut, Remove);
 * if you use those functions, just get a new pointer by calling this function again.
 *
 * If the list has no items, returns NULL.
 *
 * The pointer remains owned by the list; don't free it.
 */
VuoPoint4d *VuoListGetData_VuoPoint4d(const VuoList_VuoPoint4d list);

/**
 * Changes the @ref VuoPoint4d at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoPoint4d(const VuoList_VuoPoint4d list, const VuoPoint4d value, const unsigned long index);

/**
 * Inserts the `VuoPoint4d` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoPoint4d(const VuoList_VuoPoint4d list, const VuoPoint4d value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoPoint4d(VuoList_VuoPoint4d list, const VuoPoint4d value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoPoint4d(VuoList_VuoPoint4d list, const VuoPoint4d value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoPoint4d(VuoList_VuoPoint4d list, const unsigned long indexA, const unsigned long indexB);

#ifdef VuoPoint4d_SUPPORTS_COMPARISON
/**
 * Sorts `list`.
 */
void VuoListSort_VuoPoint4d(VuoList_VuoPoint4d list);

/**
 * Returns true if the two lists are equivalent.
 *
 * NULL lists are never equal to non-NULL lists (even empty lists).
 */
bool VuoList_VuoPoint4d_areEqual(const VuoList_VuoPoint4d a, const VuoList_VuoPoint4d b);

/**
 * Returns true if `a` < `b`.
 */
bool VuoList_VuoPoint4d_isLessThan(const VuoList_VuoPoint4d a, const VuoList_VuoPoint4d b);
#endif

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoPoint4d(VuoList_VuoPoint4d list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoPoint4d(VuoList_VuoPoint4d list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoPoint4d(VuoList_VuoPoint4d list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoPoint4d(VuoList_VuoPoint4d list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoPoint4d(VuoList_VuoPoint4d list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoPoint4d(VuoList_VuoPoint4d list);

/**
 * Removes the `VuoPoint4d` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoPoint4d(VuoList_VuoPoint4d list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoPoint4d(const VuoList_VuoPoint4d list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoPoint4d VuoList_VuoPoint4d_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoPoint4d_getJson(const VuoList_VuoPoint4d value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoPoint4d_getSummary(const VuoList_VuoPoint4d value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoPoint4d VuoList_VuoPoint4d_makeFromString(const char *str);
char * VuoList_VuoPoint4d_getString(const VuoList_VuoPoint4d value);
/// @}

/**
 * @}
 */

#endif
